<?php

namespace Facade\FlareClient\Enums;

class MessageLevels
{
    public const INFO = 'info';

    public const DEBUG = 'debug';

    public const WARNING = 'warning';

    public const ERROR = 'error';

    public const CRITICAL = 'critical';
}
