<?php

// polish

static $data = [
    'ach',
    'aj',
    'albo',
    'bardzo',
    'bez',
    'bo',
    'być',
    'ci',
    'cię',
    'ciebie',
    'co',
    'czy',
    'daleko',
    'dla',
    'dlaczego',
    'dlatego',
    'do',
    'dobrze',
    'dokąd',
    'dość',
    'dużo',
    'dwa',
    'dwaj',
    'dwie',
    'dwoje',
    'dziś',
    'dzisiaj',
    'gdyby',
    'gdzie',
    'go',
    'ich',
    'ile',
    'im',
    'inny',
    'ja',
    'ją',
    'jak',
    'jakby',
    'jaki',
    'je',
    'jeden',
    'jedna',
    'jedno',
    'jego',
    'jej',
    'jemu',
    'jeśli',
    'jest',
    'jestem',
    'jeżeli',
    'już',
    'każdy',
    'kiedy',
    'kierunku',
    'kto',
    'ku',
    'lub',
    'ma',
    'mają',
    'mam',
    'mi',
    'mną',
    'mnie',
    'moi',
    'mój',
    'moja',
    'moje',
    'może',
    'mu',
    'my',
    'na',
    'nam',
    'nami',
    'nas',
    'nasi',
    'nasz',
    'nasza',
    'nasze',
    'natychmiast',
    'nią',
    'nic',
    'nich',
    'nie',
    'niego',
    'niej',
    'niemu',
    'nigdy',
    'nim',
    'nimi',
    'niż',
    'obok',
    'od',
    'około',
    'on',
    'ona',
    'one',
    'oni',
    'ono',
    'owszem',
    'po',
    'pod',
    'ponieważ',
    'przed',
    'przedtem',
    'są',
    'sam',
    'sama',
    'się',
    'skąd',
    'tak',
    'taki',
    'tam',
    'ten',
    'to',
    'tobą',
    'tobie',
    'tu',
    'tutaj',
    'twoi',
    'twój',
    'twoja',
    'twoje',
    'ty',
    'wam',
    'wami',
    'was',
    'wasi',
    'wasz',
    'wasza',
    'wasze',
    'we',
    'więc',
    'wszystko',
    'wtedy',
    'wy',
    'żaden',
    'zawsze',
    'że',
];

$result =& $data;
unset($data);
return $result;
