import 'core-js/features/array/from';
import 'core-js/features/array/includes';
import 'core-js/features/array/flat';
import 'core-js/features/array/find';
import 'core-js/features/object/assign';
import 'core-js/features/object/entries';
import 'core-js/features/object/values';
import 'core-js/features/promise';
import 'core-js/features/string/starts-with';
import 'whatwg-fetch'
import './modules/es.element.get-attribute-names';
import './modules/es.element.matches';
import './modules/es.element.closest';
