<?php

// estonian

static $data = [
    'aga',
    'ei',
    'et',
    'ja',
    'jah',
    'kas',
    'kui',
    'kõik',
    'ma',
    'me',
    'mida',
    'midagi',
    'mind',
    'minu',
    'mis',
    'mu',
    'mul',
    'mulle',
    'nad',
    'nii',
    'oled',
    'olen',
    'oli',
    'oma',
    'on',
    'pole',
    'sa',
    'seda',
    'see',
    'selle',
    'siin',
    'siis',
    'ta',
    'te',
    'ära',
];

$result =& $data;
unset($data);
return $result;
