<?php

namespace Cornford\Googlmapper\Traits;

trait Languages
{
    /**
     * Languages.
     *
     * @var array
     */
    protected $languages = [
        'af',
        'ar-ae',
        'ar-bh',
        'ar-dz',
        'ar-eg',
        'ar-iq',
        'ar-jo',
        'ar-kw',
        'ar-lb',
        'ar-ly',
        'ar-ma',
        'ar-om',
        'ar-qa',
        'ar-sa',
        'ar-sy',
        'ar-tn',
        'ar-ye',
        'be',
        'bg',
        'ca',
        'cs',
        'da',
        'de',
        'de-at',
        'de-ch',
        'de-li',
        'de-lu',
        'el',
        'en',
        'en-au',
        'en-bz',
        'en-ca',
        'en-gb',
        'en-ie',
        'en-jm',
        'en-nz',
        'en-tt',
        'en-us',
        'en-za',
        'es',
        'es-ar',
        'es-bo',
        'es-cl',
        'es-co',
        'es-cr',
        'es-do',
        'es-ec',
        'es-gt',
        'es-hn',
        'es-mx',
        'es-ni',
        'es-pa',
        'es-pe',
        'es-pr',
        'es-py',
        'es-sv',
        'es-uy',
        'es-ve',
        'et',
        'eu',
        'fa',
        'fi',
        'fo',
        'fr',
        'fr-be',
        'fr-ca',
        'fr-ch',
        'fr-lu',
        'ga',
        'gd',
        'he',
        'hi',
        'hr',
        'hu',
        'id',
        'is',
        'it',
        'it-ch',
        'ja',
        'ji',
        'ko',
        'ko',
        'ku',
        'lt',
        'lv',
        'mk',
        'ml',
        'ms',
        'mt',
        'nl',
        'nl-be',
        'nb',
        'nn',
        'no',
        'pa',
        'pl',
        'pt',
        'pt-br',
        'rm',
        'ro',
        'ro-md',
        'ru',
        'ru-md',
        'sb',
        'sk',
        'sl',
        'sq',
        'sr',
        'sv',
        'sv-fi',
        'th',
        'tn',
        'tr',
        'ts',
        'uk',
        'ur',
        've',
        'vi',
        'xh',
        'zh-cn',
        'zh-hk',
        'zh-sg',
        'zh-tw',
        'zu'
    ];
}
