/* Force Laravel Whoops exception handler to be displayed under the debug bar */
.Whoops.container {
    z-index: 5999999;
}

div.phpdebugbar {
    font-size: 13px;
    font-family: "Lucida Grande", "Lucida Sans Unicode", "Lucida Sans", Geneva, Verdana, sans-serif;
    direction: ltr;
    text-align: left;
    z-index: 6000000;
}

div.phpdebugbar * {
    direction: ltr;
    text-align: left;
}

div.phpdebugbar-openhandler-overlay {
    z-index: 6000001;
    cursor: pointer;
}

div.phpdebugbar-openhandler {
    border: 1px solid #aaa;
    border-top: 3px solid #fa5661;
    width: 80%;
    height: 70%;
    padding: 10px;
    border-radius: 5px;
    overflow-y: scroll;
    z-index: 6000002;
    cursor: default;
}

div.phpdebugbar-openhandler .phpdebugbar-openhandler-actions > a,
div.phpdebugbar-openhandler .phpdebugbar-openhandler-actions button {
    display: inline-block;
    cursor: pointer;
    margin: 5px;
    padding: 0px 12px 2px;
    border-radius: 3px;
    border: 1px solid #ddd;
    background-color: #f5f5f5;
    color: #000;
    text-shadow: 1px 1px #fff;
    font-size: 13px;
}

div.phpdebugbar-openhandler .phpdebugbar-openhandler-actions > form {
    margin: 15px 0px 5px;
    text-transform: uppercase;
    font-size: 13px;
    font-weight: bold;
}

div.phpdebugbar-openhandler .phpdebugbar-openhandler-actions > form br {
    display: none;
}

div.phpdebugbar-openhandler .phpdebugbar-openhandler-actions > form > b {
    display: none;
}

div.phpdebugbar-openhandler .phpdebugbar-openhandler-actions > form input,
div.phpdebugbar-openhandler .phpdebugbar-openhandler-actions > form select {
    margin: 0px 10px 10px 2px;
    border: 1px solid #aaa;
    border-radius: 3px;
    padding: 3px 6px 2px;
    line-height: 16px;
}

@media (max-width: 720px) {
    div.phpdebugbar-openhandler .phpdebugbar-openhandler-actions > form select + br,
    div.phpdebugbar-openhandler .phpdebugbar-openhandler-actions > form input[name="uri"] + br {
        display: block;
    }
}

div.phpdebugbar-openhandler .phpdebugbar-openhandler-actions > form select {
    padding: 2px 5px 1px;
}

div.phpdebugbar-openhandler .phpdebugbar-openhandler-actions input[name="uri"] {
    width: 200px;
}

div.phpdebugbar-openhandler .phpdebugbar-openhandler-actions input[name="ip"] {
    width: 90px;
}

div.phpdebugbar-openhandler .phpdebugbar-openhandler-actions button {
    outline: none;
    margin: 0px 0px 10px 2px;
    padding: 5px 15px 4px;
    line-height: 12px;
}

div.phpdebugbar-openhandler table {
    margin: 15px 0px 10px;
    table-layout: auto;
    border-collapse: collapse;
    width: 100%;
}

div.phpdebugbar-openhandler table td,
div.phpdebugbar-openhandler table th {
    width: auto!important;
    text-align: left;
    border: 0px solid #bbb;
    padding: 2px 8px;
    font-size: 14px;
}

div.phpdebugbar-openhandler table th {
    text-shadow: 1px 1px #fff;
    font-size: 12px;
    text-transform: uppercase;
    padding: 5px 8px;
}

div.phpdebugbar-openhandler table th,
div.phpdebugbar-openhandler table tr:nth-child(2n) {
    background-color: #efefef;
}

div.phpdebugbar-openhandler table th:nth-child(1), div.phpdebugbar-openhandler table td:nth-child(1), /* Date */
div.phpdebugbar-openhandler table th:nth-child(2), div.phpdebugbar-openhandler table td:nth-child(2), /* Method */
div.phpdebugbar-openhandler table th:nth-child(4), div.phpdebugbar-openhandler table td:nth-child(4), /* IP */
div.phpdebugbar-openhandler table th:nth-child(5), div.phpdebugbar-openhandler table td:nth-child(5) { /* Filter */
    width: 5%!important;
    white-space: nowrap;
}

div.phpdebugbar-openhandler table th:nth-child(2), div.phpdebugbar-openhandler table td:nth-child(2), /* Method */
div.phpdebugbar-openhandler table th:nth-child(4), div.phpdebugbar-openhandler table td:nth-child(4), /* IP */
div.phpdebugbar-openhandler table th:nth-child(5), div.phpdebugbar-openhandler table td:nth-child(5) { /* Filter */
    text-align: center;
}

div.phpdebugbar-openhandler table th:nth-child(3) { /* URL */
    width: calc(100vw - 100% - 196px)!important;
}

div.phpdebugbar-openhandler table td a {
    display: block;
    white-space: nowrap;
    overflow: hidden;
    text-overflow: ellipsis;
    cursor: pointer;
}

div.phpdebugbar-resize-handle {
    display: block!important;
    height: 3px;
    margin-top: -4px;
    width: 100%;
    background: none;
    cursor: ns-resize;
    border-top: none;
    border-bottom: 0px;
    background-color: #fa5661;
}

.phpdebugbar.phpdebugbar-minimized div.phpdebugbar-resize-handle {
    cursor: default!important;
}

div.phpdebugbar-closed,
div.phpdebugbar-minimized {
    border-top-color: #ddd;
}

div.phpdebugbar code, div.phpdebugbar pre, div.phpdebugbar samp {
    background: none;
    font-family: "SFMono-Regular", Consolas, "Liberation Mono", Menlo, Courier, monospace;
    font-size: 1em;
    border: 0;
    padding: 0;
}

div.phpdebugbar .hljs {
    padding: 0;
}

div.phpdebugbar .phpdebugbar-widgets-messages .hljs > code {
    padding-bottom: 3px;
}

div.phpdebugbar code, div.phpdebugbar pre {
    color: #000;
}

div.phpdebugbar-widgets-exceptions .phpdebugbar-widgets-filename {
    margin-top: 4px;
}

div.phpdebugbar-widgets-exceptions li.phpdebugbar-widgets-list-item pre.phpdebugbar-widgets-file {
    border: 1px solid #d2d2d2;
    border-left: 2px solid #d2d2d2;
}

div.phpdebugbar-widgets-exceptions li.phpdebugbar-widgets-list-item pre.phpdebugbar-widgets-file[style="display: block;"] ~ div {
    display: block;
}

div.phpdebugbar pre.sf-dump {
    color: #000;
    outline: none;
    padding: 0px!important;
}

div.phpdebugbar-body {
    border-top: 1px solid #ddd;
}

div.phpdebugbar-header {
    min-height: 30px;
    line-height: 20px;
    padding-left: 39px;
    text-shadow: 1px 1px #FFF;
}

div.phpdebugbar-header,
a.phpdebugbar-restore-btn,
div.phpdebugbar-openhandler .phpdebugbar-openhandler-header {
    background: #f5f5f5 url(data:image/png;base64, ) no-repeat 5px 3px;
}

div.phpdebugbar-openhandler .phpdebugbar-openhandler-header {
    background-size: 20px;
    padding: 2px 0px 3px 36px;
    background-position: 9px 5px;
    border-radius: 3px;
}

div.phpdebugbar-openhandler .phpdebugbar-openhandler-header a {
    cursor: pointer;
}

a.phpdebugbar-close-btn {
    background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAA4AAAAOCAYAAAAfSC3RAAAAAXNSR0IArs4c6QAAAARnQU1BAACxjwv8YQUAAAAJcEhZcwAADsMAAA7DAcdvqGQAAAAYdEVYdFNvZnR3YXJlAHBhaW50Lm5ldCA0LjAuM4zml1AAAADDSURBVDhPxZCxCsIwFEUzuGdwCvQD7BIIcehUXDqVfGM/wsG/iG4ifkzMlRuSPLo4eeFBue8c6Iv6b4wxW557Hs0KnWa3seqDxTiOyVqbhmF4UND4Rofdruyce3rvE6bIRSo9GOI1McbLPM/vVm4l7MAQr0kpHaQsJTDE+6zrepym6SVFdNgR69M+hBTLzWCI10gJvydvBkO8ZlmWayvhJnkzGOI1+fBTCOHWPkT7YNiBId4HizxnCKy+r81uX/otSn0A7dioI/vYX+8AAAAASUVORK5CYII=) no-repeat 9px 6px;
    color : #555;
    border-right: none;
}

a.phpdebugbar-open-btn {
    background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABEAAAAOCAYAAADJ7fe0AAAAAXNSR0IArs4c6QAAAARnQU1BAACxjwv8YQUAAAAJcEhZcwAACxIAAAsSAdLdfvwAAAAHdElNRQfdCgYULwwNKp3GAAAAGHRFWHRTb2Z0d2FyZQBwYWludC5uZXQgNC4wLjOM5pdQAAAA1UlEQVQ4T2OgKpCUlOQH4vdA/B8Jv4dKEwYgDdLS0v8NDQ3/GxsbwzGIj2YoGEO1oQJkjcRgqDZUAJKwsrJ6/v//fwdiMFQbKgAZkpGR0QR0ajy60wlgRJhBXSGhpqb2CNnZhHBkZORcqBEMDFBX2BsYGGBVjAv39vZaQ41gYIC6Ygs2hbiwr6/vdqA+DqgR4CiW19bWxqoYF87Ly4uFaocAZWXlydgU4sJ2dna3ga4QgGqHAC0trY/YFOPCKSkpDVCtCAA01QaIsaYJHFgCqpVagIEBACGlF2c3r4ViAAAAAElFTkSuQmCC) no-repeat 8px 6px;
}


div.phpdebugbar-header,
div.phpdebugbar-openhandler-header {
    background-size: 21px auto;
    background-position: 9px center;
}

a.phpdebugbar-restore-btn {
    border-right-color: #ddd!important;
    height: 20px;
    width: 23px;
    background-position: center;
    background-size: 21px;
}

div.phpdebugbar-header > div > * {
    font-size: 13px;
    padding: 5px;
}

div.phpdebugbar-header .phpdebugbar-tab {
    padding: 5px 8px;
    border-left: 1px solid #ddd;
}

div.phpdebugbar-header .phpdebugbar-header-left {
    -webkit-touch-callout: none;
    -webkit-user-select: none;
    -khtml-user-select: none;
    -moz-user-select: none;
    -ms-user-select: none;
    user-select: none;
}

div.phpdebugbar .phpdebugbar-header select {
    margin: 4px 0px 0px 8px;
    padding: 2px 3px 3px 3px;
    border-radius: 3px;
    width: auto;
    cursor: pointer;
}

dl.phpdebugbar-widgets-kvlist dt,
dl.phpdebugbar-widgets-kvlist dd {
    min-height: 20px;
    line-height: 20px;
    padding: 4px 5px 5px;
    border-top: 0px;
}

dl.phpdebugbar-widgets-kvlist dd.phpdebugbar-widgets-value.phpdebugbar-widgets-pretty .phpdebugbar-widgets-code-block {
    padding: 0px 0px;
    background: transparent;
}

dl.phpdebugbar-widgets-kvlist dt {
    width: 25%;
}

dl.phpdebugbar-widgets-kvlist dd {
    margin-left: 25%;
}

ul.phpdebugbar-widgets-timeline .phpdebugbar-widgets-measure {
    height: 28px;
    line-height: 28px;
    border: none;
}

ul.phpdebugbar-widgets-timeline li span.phpdebugbar-widgets-value {
    height: 16px;
    background-color: #63abca;
    border-bottom: 2px solid #477e96;
}

ul.phpdebugbar-widgets-timeline li span.phpdebugbar-widgets-label,
ul.phpdebugbar-widgets-timeline li span.phpdebugbar-widgets-collector {
    top: 0px;
    color: #000;
    font-size: 11px;
}

ul.phpdebugbar-widgets-timeline li .phpdebugbar-widgets-value span.phpdebugbar-widgets-label {
    color: #fff;
    text-shadow: 1px 1px #000;
}

ul.phpdebugbar-widgets-timeline table.phpdebugbar-widgets-params {
    font-size: 11px;
}

div.phpdebugbar-widgets-messages div.phpdebugbar-widgets-toolbar {
    width: calc(100% - 20px);
    padding: 4px 0px 4px;
    height: 20px;
    border: 1px solid #ddd;
    border-bottom: 0px;
    background-color: #e8e8e8;
    border-radius: 5px 5px 0px 0px;
}

div.phpdebugbar-widgets-messages div.phpdebugbar-widgets-toolbar input {
    width: calc(100% - 48px);
    margin-left: 0px;
    border-radius: 3px;
    padding: 2px 6px;
    height: 15px;
}

div.phpdebugbar-widgets-messages li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-label,
div.phpdebugbar-widgets-messages li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-collector {
    padding: 1px 0px 0px 10px;
    margin: 0px;
    text-transform: uppercase;
    font-style: normal;
    color: #333;
}

.phpdebugbar-widgets-toolbar i.phpdebugbar-fa.phpdebugbar-fa-search {
    position: relative;
    top: -1px;
    padding: 0px 10px;
}

div.phpdebugbar-widgets-messages div.phpdebugbar-widgets-toolbar a.phpdebugbar-widgets-filter,
div.phpdebugbar-widgets-messages div.phpdebugbar-widgets-toolbar a.phpdebugbar-widgets-filter.phpdebugbar-widgets-excluded {
    position: relative;
    top: -48px;
    display: inline-block;
    background-color: #6d6d6d;
    margin-left: 3px;
    border-radius: 3px;
    padding: 5px 8px 4px;
    text-transform: uppercase;
    font-size: 10px;
    text-shadow: 1px 1px #585858;
    transition: background-color .25s linear 0s, color .25s linear 0s;
    color: #FFF;

    -webkit-touch-callout: none;
    -webkit-user-select: none;
    -khtml-user-select: none;
    -moz-user-select: none;
    -ms-user-select: none;
    user-select: none;
}

div.phpdebugbar-widgets-messages div.phpdebugbar-widgets-toolbar a.phpdebugbar-widgets-filter[rel="info"],
div.phpdebugbar-widgets-messages div.phpdebugbar-widgets-toolbar a.phpdebugbar-widgets-filter.phpdebugbar-widgets-excluded[rel="info"] {
    background-color: #5896e2;
}

div.phpdebugbar-widgets-messages div.phpdebugbar-widgets-toolbar a.phpdebugbar-widgets-filter[rel="error"],
div.phpdebugbar-widgets-messages div.phpdebugbar-widgets-toolbar a.phpdebugbar-widgets-filter.phpdebugbar-widgets-excluded[rel="error"] {
    background-color: #fa5661;
}

div.phpdebugbar-widgets-messages div.phpdebugbar-widgets-toolbar a.phpdebugbar-widgets-filter[rel="warning"],
div.phpdebugbar-widgets-messages div.phpdebugbar-widgets-toolbar a.phpdebugbar-widgets-filter.phpdebugbar-widgets-excluded[rel="warning"] {
    background-color: #f99400;
}

div.phpdebugbar-widgets-messages div.phpdebugbar-widgets-toolbar a.phpdebugbar-widgets-filter:hover {
    color: #FFF;
    opacity: 0.85;
}

div.phpdebugbar-widgets-messages div.phpdebugbar-widgets-toolbar a.phpdebugbar-widgets-filter.phpdebugbar-widgets-excluded {
    opacity: 0.45;
}

a.phpdebugbar-tab:hover,
span.phpdebugbar-indicator:hover,
a.phpdebugbar-indicator:hover,
a.phpdebugbar-close-btn:hover,
a.phpdebugbar-minimize-btn:hover,
a.phpdebugbar-maximize-btn:hover,
a.phpdebugbar-open-btn:hover {
    background-color: #ebebeb;
    /* transition: background-color .25s linear 0s, color .25s linear 0s; */
}

a.phpdebugbar-minimize-btn,
a.phpdebugbar-maximize-btn {
    width: 28px!important;
}

a.phpdebugbar-tab.phpdebugbar-active {
    background: #fa5661;
    background-image: none;
    color: #fff !important;
    text-shadow: 1px 1px #bf3039;
}

a.phpdebugbar-tab.phpdebugbar-active span.phpdebugbar-badge {
    background-color: white;
    color: #fa5661;
    text-shadow: 1px 1px #ebebeb;
}

a.phpdebugbar-tab span.phpdebugbar-badge {
    vertical-align: 0px;
    padding: 2px 8px 3px 8px;
    text-align: center;
    background: #fa5661;
    font-size: 11px;
    font-family: monospace;
    color: #fff;
    text-shadow: 1px 1px #bf3039;
    border-radius: 10px;
    top: -1px;
    position: relative;
}

.phpdebugbar-indicator {
    cursor: text;
}

.phpdebugbar-indicator span.phpdebugbar-tooltip,
div.phpdebugbar-mini-design a.phpdebugbar-tab:hover span.phpdebugbar-text {
    border: none;
    border-radius: 5px;
    background: #f5f5f5;
    font-size: 12px;
    width: auto;
    white-space: nowrap;
    padding: 2px 18px;
    text-shadow: none;

    -webkit-touch-callout: none;
    -webkit-user-select: none;
    -khtml-user-select: none;
    -moz-user-select: none;
    -ms-user-select: none;
    user-select: none;
}

div.phpdebugbar-mini-design a.phpdebugbar-tab:hover span.phpdebugbar-text {
    left: 0px;
    right: auto;
}

.phpdebugbar-widgets-toolbar > .fa {
    width: 25px;
    font-size: 15px;
    color: #555;
    text-align: center;
}

ul.phpdebugbar-widgets-list li.phpdebugbar-widgets-list-item {
    padding: 7px 10px;
    border: none;
    font-family: inherit;
    overflow: visible;
}

ul.phpdebugbar-widgets-list li.phpdebugbar-widgets-list-item:hover,
ul.phpdebugbar-widgets-timeline li:hover {
    background-color: initial;
}

.phpdebugbar-widgets-sqlqueries ul.phpdebugbar-widgets-list li.phpdebugbar-widgets-list-item {
    display: flex;
    flex-wrap: wrap;
}

.phpdebugbar-widgets-templates ul.phpdebugbar-widgets-list li.phpdebugbar-widgets-list-item {
    display: block;
}

.phpdebugbar-widgets-templates ul.phpdebugbar-widgets-list li.phpdebugbar-widgets-list-item,
.phpdebugbar-widgets-mails ul.phpdebugbar-widgets-list li.phpdebugbar-widgets-list-item {
    line-height: 15px;
}

.phpdebugbar-widgets-mails ul.phpdebugbar-widgets-list li.phpdebugbar-widgets-list-item {
    cursor: pointer;
    display: block;
}

.phpdebugbar-widgets-mails ul.phpdebugbar-widgets-list li.phpdebugbar-widgets-list-item .phpdebugbar-widgets-subject {
    display: inline-block;
    margin-right: 15px;
}

.phpdebugbar-widgets-mails ul.phpdebugbar-widgets-list li.phpdebugbar-widgets-list-item .phpdebugbar-widgets-headers {
    margin: 10px 0px;
    padding: 7px 10px;
    border-left: 2px solid #ddd;
    line-height: 17px;
}

.phpdebugbar-widgets-list .phpdebugbar-widgets-list-item .phpdebugbar-widgets-name {
    height: 15px;
}

.phpdebugbar-widgets-sql.phpdebugbar-widgets-name {
    font-weight: bold;
}

ul.phpdebugbar-widgets-list li.phpdebugbar-widgets-list-item .phpdebugbar-widgets-sql {
    flex: 1;
    margin-right: 5px;
}

ul.phpdebugbar-widgets-list li.phpdebugbar-widgets-list-item .phpdebugbar-widgets-duration {
    margin-left: auto;
    margin-right: 5px;
}

ul.phpdebugbar-widgets-list li.phpdebugbar-widgets-list-item .phpdebugbar-widgets-database {
    margin-left: auto;
}

ul.phpdebugbar-widgets-list li.phpdebugbar-widgets-list-item .phpdebugbar-widgets-stmt-id {
    margin-left: auto;
    margin-right: 5px;
}

ul.phpdebugbar-widgets-list li.phpdebugbar-widgets-list-item table.phpdebugbar-widgets-params {
    background-color: #fdfdfd;
    margin: 10px 0px;
    font-size: 12px;
    border-left: 2px solid #cecece;
}

div.phpdebugbar-widgets-templates table.phpdebugbar-widgets-params th,
div.phpdebugbar-widgets-templates table.phpdebugbar-widgets-params td {
    padding: 1px 10px!important;
}

div.phpdebugbar-widgets-templates table.phpdebugbar-widgets-params th {
    padding: 2px 10px!important;
    background-color: #efefef;
}

div.phpdebugbar-widgets-sqlqueries table.phpdebugbar-widgets-params td.phpdebugbar-widgets-name {
    width: auto;
}

div.phpdebugbar-widgets-sqlqueries table.phpdebugbar-widgets-params td.phpdebugbar-widgets-name .phpdebugbar-fa {
    position: relative;
    top: 1px;
    margin-left: 3px;
}

ul.phpdebugbar-widgets-list li.phpdebugbar-widgets-list-item:nth-child(even) {
    background-color: #f5f5f5;
}

div.phpdebugbar-widgets-messages li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-value {
    display: inline-flex;
}

div.phpdebugbar-widgets-messages li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-value:before {
    font-family: PhpDebugbarFontAwesome;
    content: "\f005";
    color: #333;
    font-size: 13px;
    margin-right: 8px;
    float: left;
}

div.phpdebugbar-widgets-messages li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-value.phpdebugbar-widgets-info {
    color: #1299DA;
}

div.phpdebugbar-widgets-messages li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-value.phpdebugbar-widgets-info:before {
    font-family: PhpDebugbarFontAwesome;
    content: "\f05a";
    color: #5896e2;
    font-size: 15px;
}

div.phpdebugbar-widgets-messages li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-value.phpdebugbar-widgets-error {
    color: #e74c3c;
}

div.phpdebugbar-widgets-messages li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-value.phpdebugbar-widgets-error:before {
    color: #fa5661;
    font-size: 15px;
}

div.phpdebugbar-widgets-messages li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-value.phpdebugbar-widgets-warning:before {
    color: #FF9800;
    font-size: 13px;
}

div.phpdebugbar-widgets-messages .phpdebugbar-widgets-value.phpdebugbar-widgets-warning {
    color: #FF9800;
}

div.phpdebugbar-widgets-messages li.phpdebugbar-widgets-list-item pre.sf-dump {
    display: inline-block;
    position: relative;
    top: -1px;
}

div.phpdebugbar-widgets-sqlqueries {
    line-height: 20px;
}


div.phpdebugbar-panel div.phpdebugbar-widgets-status {
    padding: 9px 20px!important;
    width: calc(100% - 20px);
    margin-left: -10px;
    margin-top: -10px;
    line-height: 11px!important;
    font-weight: bold!important;
    background: #f5f5f5!important;
    border-bottom: 1px solid #cecece!important;
}

div.phpdebugbar-panel div.phpdebugbar-widgets-status > * {
    color: #383838!important;
}

div.phpdebugbar-panel div.phpdebugbar-widgets-status > span:first-child:before {
    font-family: PhpDebugbarFontAwesome;
    content: "\f05a";
    color: #737373;
    text-shadow: 1px 1px #fff;
    font-size: 14px;
    position: relative;
    top: 1px;
    margin-right: 8px;
}

div.phpdebugbar-widgets-sqlqueries table.phpdebugbar-widgets-params th,
div.phpdebugbar-widgets-sqlqueries table.phpdebugbar-widgets-params td {
    padding: 4px 10px;
}

div.phpdebugbar-widgets-sqlqueries table.phpdebugbar-widgets-params th {
    background-color: #efefef;
}

div.phpdebugbar-widgets-sqlqueries table.phpdebugbar-widgets-params td.phpdebugbar-widgets-name {
    text-align: right;
    vertical-align: top;
    white-space: nowrap;
}

div.phpdebugbar-widgets-sqlqueries table.phpdebugbar-widgets-params td.phpdebugbar-widgets-value {
    text-align: left;
}

div.phpdebugbar-widgets-templates .phpdebugbar-widgets-list-item table.phpdebugbar-widgets-params {
    width: auto!important;
}

ul.phpdebugbar-widgets-list ul.phpdebugbar-widgets-table-list {
    text-align: left;
}

.phpdebugbar-text-muted {
    color: #888;
}

ul.phpdebugbar-widgets-cache a.phpdebugbar-widgets-forget {
    float: right;
    font-size: 12px;
    padding: 0 4px;
    background: #fa5661;
    margin: 0 2px;
    border-radius: 4px;
    color: #fff;
    text-decoration: none;
    line-height: 1.5rem;
}

a.phpdebugbar-tab i {
    line-height: 20px;
}

div.phpdebugbar-mini-design a.phpdebugbar-tab {
    border-right: none;
}

div.phpdebugbar-header-right > a {
    height: 20px;
    width: 20px;
    background-position: center;
}

div.phpdebugbar-header-right .phpdebugbar-indicator > i.phpdebugbar-fa {
    vertical-align: baseline;
    margin-top: 2px;
}

div.phpdebugbar-panel {
    width: calc(100% - 20px);
    height: calc(100% - 20px);
    padding: 10px;
}

div.phpdebugbar-panel table {
    margin: 10px 0px!important;
    width: 100%!important;
}

div.phpdebugbar-panel table .phpdebugbar-widgets-name {
    font-size: 13px;
}

dl.phpdebugbar-widgets-kvlist > :nth-child(4n-1),
dl.phpdebugbar-widgets-kvlist > :nth-child(4n) {
    background-color: #f5f5f5;
}

.phpdebugbar pre.sf-dump:after {
    clear: none!important;
}

div.phpdebugbar-widgets-exceptions li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-message {
    color: #dd1044;
}

div.phpdebugbar-widgets-exceptions li.phpdebugbar-widgets-list-item > div {
    display: none;
}


div.phpdebugbar-widgets-sqlqueries span.phpdebugbar-widgets-database:before,
div.phpdebugbar-widgets-sqlqueries span.phpdebugbar-widgets-duration:before,
div.phpdebugbar-widgets-sqlqueries span.phpdebugbar-widgets-memory:before,
div.phpdebugbar-widgets-sqlqueries span.phpdebugbar-widgets-row-count:before,
div.phpdebugbar-widgets-sqlqueries span.phpdebugbar-widgets-copy-clipboard:before,
div.phpdebugbar-widgets-sqlqueries span.phpdebugbar-widgets-stmt-id:before,
div.phpdebugbar-widgets-templates span.phpdebugbar-widgets-param-count:before {
    margin-right: 6px!important;
}

.phpdebugbar-widgets-list-item .phpdebugbar-widgets-bg-measure {
    position: absolute;
    top: 0;
    width: 100%;
    height: 100%;
    overflow: hidden;
    pointer-events: none;
}

.phpdebugbar-widgets-bg-measure .phpdebugbar-widgets-value {
    position: absolute;
    height: 100%;
    opacity: 0.2;
    background: red;
}
