<?php

// autoload_namespaces.php @generated by Composer

$vendorDir = dirname(dirname(__FILE__));
$baseDir = dirname($vendorDir);

return array(
    'URLify' => array($vendorDir . '/jbroadway/urlify'),
    'Orangehill\\Iseed' => array($vendorDir . '/orangehill/iseed/src'),
    'Mockery' => array($vendorDir . '/mockery/mockery/library'),
    'Milon\\Barcode' => array($vendorDir . '/milon/barcode/src'),
);
