<?php

// ukrainian

static $data = [
    'a',
    'б',
    'в',
    'г',
    'е',
    'ж',
    'з',
    'м',
    'т',
    'у',
    'я',
    'є',
    'і',
    'аж',
    'ви',
    'де',
    'до',
    'за',
    'зі',
    'ми',
    'на',
    'не',
    'ну',
    'нх',
    'ні',
    'по',
    'та',
    'ти',
    'то',
    'ту',
    'ті',
    'це',
    'цю',
    'ця',
    'ці',
    'чи',
    'ще',
    'що',
    'як',
    'їй',
    'їм',
    'їх',
    'її',
    'або',
    'але',
    'ало',
    'без',
    'був',
    'вам',
    'вас',
    'ваш',
    'вже',
    'все',
    'всю',
    'вся',
    'від',
    'він',
    'два',
    'дві',
    'для',
    'ким',
    'мож',
    'моя',
    'моє',
    'мої',
    'міг',
    'між',
    'мій',
    'над',
    'нам',
    'нас',
    'наш',
    'нею',
    'неї',
    'них',
    'ніж',
    'ній',
    'ось',
    'при',
    'про',
    'під',
    'пір',
    'раз',
    'рік',
    'сам',
    'сих',
    'сім',
    'так',
    'там',
    'теж',
    'тим',
    'тих',
    'той',
    'тою',
    'три',
    'тут',
    'хоч',
    'хто',
    'цей',
    'цим',
    'цих',
    'час',
    'щоб',
    'яка',
    'які',
    'адже',
    'буде',
    'буду',
    'будь',
    'була',
    'були',
    'було',
    'бути',
    'вами',
    'ваша',
    'ваше',
    'ваші',
    'весь',
    'вниз',
    'вона',
    'вони',
    'воно',
    'всею',
    'всім',
    'всіх',
    'втім',
    'геть',
    'далі',
    'двох',
    'день',
    'дуже',
    'зате',
    'його',
    'йому',
    'каже',
    'кого',
    'коли',
    'кому',
    'крім',
    'куди',
    'лише',
    'люди',
    'мало',
    'мати',
    'мене',
    'мені',
    'миру',
    'мною',
    'може',
    'нами',
    'наша',
    'наше',
    'наші',
    'ними',
    'ніби',
    'один',
    'поки',
    'пора',
    'рано',
    'року',
    'році',
    'сама',
    'саме',
    'саму',
    'самі',
    'свою',
    'своє',
    'свої',
    'себе',
    'собі',
    'став',
    'суть',
    'така',
    'таке',
    'такі',
    'твоя',
    'твоє',
    'твій',
    'тебе',
    'тими',
    'тобі',
    'того',
    'тоді',
    'тому',
    'туди',
    'хоча',
    'хіба',
    'цими',
    'цієї',
    'часу',
    'чого',
    'чому',
    'який',
    'яких',
    'якої',
    'якщо',
    'ім\'я',
    'інша',
    'інше',
    'інші',
    'буває',
    'будеш',
    'більш',
    'вгору',
    'вміти',
    'внизу',
    'вісім',
    'давно',
    'даром',
    'добре',
    'довго',
    'друго',
    'дякую',
    'життя',
    'зараз',
    'знову',
    'какая',
    'кожен',
    'кожна',
    'кожне',
    'кожні',
    'краще',
    'ледве',
    'майже',
    'менше',
    'могти',
    'можна',
    'назад',
    'немає',
    'нижче',
    'нього',
    'однак',
    'п\'ять',
    'перед',
    'поруч',
    'потім',
    'проти',
    'після',
    'років',
    'самим',
    'самих',
    'самій',
    'свого',
    'своєї',
    'своїх',
    'собою',
    'справ',
    'такий',
    'також',
    'тепер',
    'тисяч',
    'тобою',
    'треба',
    'трохи',
    'усюди',
    'усіма',
    'хочеш',
    'цього',
    'цьому',
    'часто',
    'через',
    'шість',
    'якого',
    'іноді',
    'інший',
    'інших',
    'багато',
    'будемо',
    'будете',
    'будуть',
    'більше',
    'всього',
    'всьому',
    'далеко',
    'десять',
    'досить',
    'другий',
    'дійсно',
    'завжди',
    'звідси',
    'зовсім',
    'кругом',
    'кілька',
    'людина',
    'можуть',
    'навіть',
    'навіщо',
    'нагорі',
    'небудь',
    'низько',
    'ніколи',
    'нікуди',
    'нічого',
    'обидва',
    'одного',
    'однієї',
    'п\'ятий',
    'перший',
    'просто',
    'раніше',
    'раптом',
    'самими',
    'самого',
    'самому',
    'сказав',
    'скрізь',
    'сьомий',
    'третій',
    'тільки',
    'хотіти',
    'чотири',
    'чудово',
    'шостий',
    'близько',
    'важлива',
    'важливе',
    'важливі',
    'вдалині',
    'восьмий',
    'говорив',
    'дев\'ять',
    'десятий',
    'зайнята',
    'зайнято',
    'зайняті',
    'занадто',
    'значить',
    'навколо',
    'нарешті',
    'нерідко',
    'повинно',
    'посеред',
    'початку',
    'пізніше',
    'сказала',
    'сказати',
    'скільки',
    'спасибі',
    'частіше',
    'важливий',
    'двадцять',
    'дев\'ятий',
    'зазвичай',
    'зайнятий',
    'звичайно',
    'здається',
    'найбільш',
    'не можна',
    'недалеко',
    'особливо',
    'потрібно',
    'спочатку',
    'сьогодні',
    'численна',
    'численне',
    'численні',
    'відсотків',
    'двадцятий',
    'звідусіль',
    'мільйонів',
    'нещодавно',
    'прекрасно',
    'четвертий',
    'численний',
    'будь ласка',
    'дванадцять',
    'одинадцять',
    'сімнадцять',
    'тринадцять',
    'безперервно',
    'дванадцятий',
    'одинадцятий',
    'одного разу',
    'п\'ятнадцять',
    'сімнадцятий',
    'тринадцятий',
    'шістнадцять',
    'вісімнадцять',
    'п\'ятнадцятий',
    'чотирнадцять',
    'шістнадцятий',
    'вісімнадцятий',
    'дев\'ятнадцять',
    'чотирнадцятий',
    'дев\'ятнадцятий',
];

$result =& $data;
unset($data);
return $result;
